package Plugins::Podcasts;

use Data::Dumper;
use strict;
use LWP::UserAgent;
use HTML::Entities;
use XML::Simple;

###############################################################################
sub init {
    main::plugin_register("Podcasts", "Experimental Podcast Player", "/Podcasts", \&Plugins::Podcasts::html);
    main::http_register_handler("/Podcasts/index",   \&Plugins::Podcasts::index);
    main::http_register_handler("/Podcasts/add",     \&Plugins::Podcasts::add);
    main::add_timeout (time()+10, \&Plugins::Podcasts::timer);
}

###############################################################################
sub quit {
}

###############################################################################
sub build_zone_menu {
    my ($match) = @_;

    my $menu = "";
    foreach my $zone (keys %main::ZONES) {
        if (defined $match && ($zone eq $match)) {
            $menu .= "<OPTION VALUE=$zone SELECTED>";
        } else {
            $menu .= "<OPTION VALUE=$zone>";
        }
        $menu .= $main::ZONES{$zone}->{ZoneName} . "\n";
    }
    return $menu
}

###############################################################################
sub html {
    my ($c, $r, $diskpath, $template) = @_;

    my %qf = $r->uri->query_form;

    $template->param("PODCASTS_ZONEMENU" => build_zone_menu($qf{zone}));

    my @folder_loop_data = ();
    foreach my $id (sort (keys %{$main::PREFS{Podcasts}->{casts}})) {
        my %folder_row_data;

        next if (! $Plugins::Podcasts{$id}->{xml});

        my $x = $Plugins::Podcasts{$id}->{xml};
        $folder_row_data{FOLDER_NAME} = $x->{channel}->{title};
        $folder_row_data{FOLDER_DESC} = decode_entities($x->{channel}->{description});
        $folder_row_data{FOLDER_ID}   = $id;
        $folder_row_data{FOLDER_EXPANDED} = defined $main::PREFS{Podcasts}->{casts}->{$id}->{expanded};
        $folder_row_data{FOLDER_NOAUTOPLAYLIST} = defined $main::PREFS{Podcasts}->{casts}->{$id}->{noauto};

        my @item_loop_data = ();
        foreach my $item (@{$x->{channel}->{item}}) {
            my %item_row_data;

            $item_row_data{ITEM_ZONE} = $qf{zone};
            $item_row_data{ITEM_NAME} = $item->{title};
            $item_row_data{ITEM_DESC} = decode_entities($item->{description});
            $item_row_data{ITEM_URL}  = $item->{enclosure}->{url};
            push(@item_loop_data, \%item_row_data);
        }
        $folder_row_data{PODCASTS_ITEM_LOOP} = \@item_loop_data;

        push(@folder_loop_data, \%folder_row_data);
    }

    $template->param("PODCASTS_FOLDER_LOOP" => \@folder_loop_data);
    if (defined $Plugins::Podcasts::Err) {
        $template->param("PODCASTS_ERR" => $Plugins::Podcasts::Err);
        undef $Plugins::Podcasts::Err;
    }
}

###############################################################################
sub index {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;
    my $zonearg = "";
    if (defined $qf{zone} && $qf{zone} ne "") {
        $zonearg = "?zone=$qf{zone}";
    }
    
    if (defined $qf{refresh} && $qf{refresh} == 1) {
        foreach my $id (sort (keys %{$main::PREFS{Podcasts}->{casts}})) {
            delete $Plugins::Podcasts{$id}->{xml};
        }
        dofetch(100);
    }

    if (defined $qf{podcastdel} &&
        defined $main::PREFS{Podcasts}->{casts}->{$qf{podcastdel}}) {
        delete $main::PREFS{Podcasts}->{casts}->{$qf{podcastdel}};
    }
    
    if (defined $qf{podcastplay}) {
        my $url = decode_entities($qf{podcastplay});
        main::Log(3, "Play: $url on Zone: $qf{zone}");

        if  ($zonearg eq "") {
            $Plugins::Podcasts::Err = "<H1>Please select a zone.<P></H1>";
        } else  {
            main::upnp_avtransport_set_uri($qf{zone}, $url, "");
            main::upnp_avtransport_play($qf{zone});
        }
    }

    if (defined $qf{podcastadd}) {
        my $url = decode_entities($qf{podcastadd});
        main::Log(3, "Add: $url on Zone: $qf{zone}");

        if  ($zonearg eq "") {
            $Plugins::Podcasts::Err = "<H1>Please select a zone.<P></H1>";
        } else  {
            main::upnp_avtransport_add_uri($qf{zone}, $url, "");
        }
    }

    if (defined $qf{podcastautoon}) {
        delete $main::PREFS{Podcasts}->{casts}->{$qf{podcastautoon}}->{noauto};
        main::sonos_prefsdb_save();
    } 

    if (defined $qf{podcastautooff}) {
        $main::PREFS{Podcasts}->{casts}->{$qf{podcastautooff}}->{noauto} = 1;
        main::sonos_prefsdb_save();
    } 

    if (defined $qf{podcastcollapse}) {
        delete $main::PREFS{Podcasts}->{casts}->{$qf{podcastcollapse}}->{expanded};
        main::sonos_prefsdb_save();
    } 

    if (defined $qf{podcastexpand}) {
        $main::PREFS{Podcasts}->{casts}->{$qf{podcastexpand}}->{expanded} = 1;
        main::sonos_prefsdb_save();
    } 

    $c->send_redirect(main::http_base_url($r) . "/Podcasts/index.html$zonearg", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
    return;

}

###############################################################################
sub add {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;

    if (!defined $qf{url} || $qf{url} eq "") {
        $Plugins::Podcasts::Err = "<H1>Please enter a valid url.<P></H1>";
        $c->send_redirect(main::http_base_url($r) . "/Podcasts/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
        $c->force_last_request;
        return;
    }
    $main::PREFS{Podcasts}->{maxid}++;
    my $id  = $main::PREFS{Podcasts}->{maxid};
    $main::PREFS{Podcasts}->{casts}->{$id}->{url} = $qf{url};

    main::sonos_prefsdb_save();

    $c->send_redirect(main::http_base_url($r) . "/Podcasts/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
    return;
}
###############################################################################
sub dofetch {
    my ($maxfetch) = @_;

    my $changed = 0;
    my $fetched = 0;

    foreach my $id (sort (keys %{$main::PREFS{Podcasts}->{casts}})) {
        if (! $Plugins::Podcasts{$id}->{xml} || ($Plugins::Podcasts{$id}->{time} + 2*60*60 < time()) ) {

            main::Log(3, "Updating " . $main::PREFS{Podcasts}->{casts}->{$id}->{url});
            $Plugins::Podcasts{$id}->{time} = time();
            my $response = $Plugins::LastFM::userAgent->get($main::PREFS{Podcasts}->{casts}->{$id}->{url});
            next if (!$response->is_success());
            next if ($response->content eq "");
            $Plugins::Podcasts{$id}->{xml} = XMLin($response->content, forcearray => ["item"]);
            my $x = $Plugins::Podcasts{$id}->{xml};

            next if ($main::PREFS{Podcasts}->{casts}->{$id}->{noauto});

            my $num = 0;
            foreach my $item (@{$x->{channel}->{item}}) {
                my $guid;

                if (defined $item->{guid}) {
                    if (ref($item->{guid}) eq "HASH" && $item->{guid}->{isPermaLink}) {
                        $guid = $item->{guid}->{content};
                    } else {
                        $guid = $item->{guid};
                    }
                } else {
                    next;
                }
                next if (defined $main::PREFS{Podcasts}->{casts}->{$id}->{guids}->{$guid});
                next if (!defined $item->{enclosure} || !defined $item->{enclosure}->{url});
                $num++;
                $main::PREFS{Podcasts}->{casts}->{$id}->{guids}->{$guid} = localtime();
                $changed = 1;

                my $ctitle = decode_entities($x->{channel}->{title});
                my $ititle = decode_entities($item->{title});
                my $stitle = "Cast: " . $ctitle . " " . $ititle;
                if (length($stitle) > 47) {
                    $stitle = "Cast: " . substr($ctitle, 0, 14) . " " . $ititle;
                }

                if (length($stitle) > 47) {
                    $stitle = substr($stitle, 0, 47);
                }

                # Add at most 5 things per podcast
                if ($num < 5) {
                    my $result = main::sonos_add_radio($stitle, $item->{enclosure}->{url});
                }
            }
            $fetched++;
        }
        last if ($fetched > $maxfetch);
    }

    main::sonos_prefsdb_save() if ($changed);

    return $fetched;
}

###############################################################################
sub timer {
    main::add_timeout (time()+5, \&Plugins::Podcasts::timer);
    dofetch(1);
}


1;
